# -*- coding: utf-8 -*-
"""
    wakadump.compat
    ~~~~~~~~~~~~~~~

    For working with Python2 and Python3.

    :copyright: (c) 2015 Alan Hamlett.
    :license: BSD, see LICENSE for more details.
"""

import codecs
import sys


is_py2 = (sys.version_info[0] == 2)
is_py3 = (sys.version_info[0] == 3)


if is_py2:

    def u(text):
        try:
            return text.decode('utf-8')
        except:
            try:
                return unicode(text)
            except:
                return text
    open = codecs.open
    basestring = basestring


elif is_py3:

    def u(text):
        if isinstance(text, bytes):
            return text.decode('utf-8')
        return str(text)
    open = open
    basestring = (str, bytes)


try:
    from importlib import import_module
except ImportError:
    def _resolve_name(name, package, level):
        """Return the absolute name of the module to be imported."""
        if not hasattr(package, 'rindex'):
            raise ValueError("'package' not set to a string")
        dot = len(package)
        for x in xrange(level, 1, -1):
            try:
                dot = package.rindex('.', 0, dot)
            except ValueError:
                raise ValueError("attempted relative import beyond top-level "
                                 "package")
        return "%s.%s" % (package[:dot], name)

    def import_module(name, package=None):
        """Import a module.
        The 'package' argument is required when performing a relative import.
        It specifies the package to use as the anchor point from which to
        resolve the relative import to an absolute import.
        """
        if name.startswith('.'):
            if not package:
                raise TypeError("relative imports require the 'package' "
                                + "argument")
            level = 0
            for character in name:
                if character != '.':
                    break
                level += 1
            name = _resolve_name(name[level:], package, level)
        __import__(name)
        return sys.modules[name]



# -*- coding: utf-8 -*-
"""
    wakadump.cli
    ~~~~~~~~~~~~

    Command-line entry point.

    :copyright: (c) 2015 Alan Hamlett.
    :license: BSD, see LICENSE for more details.
"""


import click
import simplejson as json

from .__about__ import __version__
from .compat import import_module


def check_output(ctx, param, value):
    if value == 'keen.io':
        ctx.params['project_id'] = click.prompt('keen.io Project ID',
                                                type=click.STRING)
        ctx.params['write_key'] = click.prompt('keen.io project Write Key',
                                               type=click.STRING,
                                               hide_input=True)
    elif value == 'csv':
        ctx.params['output_file'] = click.prompt('Output csv file',
                                                type=click.File('w'))
    return value


def make_module_name(module_name):
    return module_name.replace('.', '').replace('-', '')


@click.command()
@click.option('--input', help='path to json data dump from wakatime.com',
              type=click.File('r'), required=True)
@click.option('--output', type=click.Choice(['keen.io', 'csv']), required=True,
              help='export format', callback=check_output)
@click.version_option(__version__)
def main(input, output, **kwargs):
    data = json.loads(input.read())

    module_name = make_module_name(output)
    module = import_module('.formats.%s' % module_name, package=__package__)
    formatter = getattr(module, 'Formatter')(data, **kwargs)
    formatter.run()
    click.echo('Complete.')



# -*- coding: utf-8 -*-
"""
    wakadump.formats.csv
    ~~~~~~~~~~~~~~~~~~~~

    Exports logged time to a CSV file.

    :copyright: (c) 2015 Alan Hamlett.
    :license: BSD, see LICENSE for more details.
"""


import click
import unicodecsv


class Formatter(object):

    def __init__(self, data, output_file=None):
        self.data = data
        self.output_file = output_file

    def run(self):
        w = unicodecsv.writer(self.output_file, encoding='utf-8')

        self._get_column_names()
        w.writerow(self.columns)

        with click.progressbar(self.data['days'],
                               label='Exporting to CSV file...',
                               fill_char=click.style('#', fg='blue')) as days:
            for day in days:
                data = [
                    day['date'],
                    day['grand_total']['total_seconds'],
                ]
                self._add_data_for_columns(data, self.projects, day['projects'])
                self._add_data_for_columns(data, self.entities, day['entities'])
                self._add_data_for_columns(data, self.languages, day['languages'])
                self._add_data_for_columns(data, self.editors, day['editors'])
                self._add_data_for_columns(data, self.operating_systems, day['operating_systems'])
                w.writerow(data)

    def _get_column_names(self):
        self.projects = {}
        self.entities = {}
        self.languages = {}
        self.editors = {}
        self.operating_systems = {}

        with click.progressbar(self.data['days'],
                               label='Getting column names...',
                               fill_char=click.style('#', fg='blue')) as days:
            for day in days:
                self._find_columns(self.projects, day['projects'])
                self._find_columns(self.entities, day['entities'])
                self._find_columns(self.languages, day['languages'])
                self._find_columns(self.editors, day['editors'])
                self._find_columns(self.operating_systems, day['operating_systems'])

        self.projects = self.projects.keys()
        self.entities = self.entities.keys()
        self.languages = self.languages.keys()
        self.editors = self.editors.keys()
        self.operating_systems = self.operating_systems.keys()

        self.columns = [
            'DATE',
            'TOTAL LOGGED SECONDS',
        ]
        self._add_columns('PROJECTS', self.projects)
        self._add_columns('FILES', self.entities)
        self._add_columns('LANGUAGES', self.languages)
        self._add_columns('EDITORS', self.editors)
        self._add_columns('OPERATING SYSTEMS', self.operating_systems)

    def _find_columns(self, existing, items):
        for item in items:
            if item['name'] not in existing:
                existing[item['name']] = True

    def _add_columns(self, name, data):
        self.columns.append(name)
        self.columns.extend(data)

    def _add_data_for_columns(self, existing_data, column_names, column_data):
        data = {}
        for item in column_data:
            data[item['name']] = item['total_seconds']
        existing_data.append('')
        for name in column_names:
            existing_data.append(data.get(name, 0))



# -*- coding: utf-8 -*-
"""
    wakadump
    ~~~~~~~~

    Command line tool for converting WakaTime data dump files into various formats.

    http://wakatime.com

    :copyright: (c) 2015 Alan Hamlett.
    :license: BSD, see LICENSE for more details.
"""



# -*- coding: utf-8 -*-
"""
    wakadump.formats.keenio
    ~~~~~~~~~~~~~~~~~~~~~~~

    Exports logged time to a Keen.io project.

    :copyright: (c) 2015 Alan Hamlett.
    :license: BSD, see LICENSE for more details.
"""


import click
import pytz
from datetime import datetime
from keen.client import KeenClient


class Formatter(object):

    def __init__(self, data, project_id=None, write_key=None):
        self.data = data
        self.project_id = project_id
        self.write_key = write_key

    def append_event(self, dt, event_type, values):
        timestamp = dt.astimezone(pytz.utc)
        event = {
            'keen': {
                'timestamp': timestamp.strftime('%Y-%m-%dT%H:%M:%S.000Z'),
            },
            'type': event_type,
            'weekday': dt.strftime('%A'),
            'weekday_number': dt.strftime('%w'),
            'day': dt.strftime('%d'),
            'month': dt.strftime('%m'),
            'year': dt.strftime('%Y'),
        }
        event.update(values)
        self.events.append(event)

    def run(self):
        keen_client = KeenClient(
            project_id=self.project_id,
            write_key=self.write_key,
        )

        timezone = pytz.timezone(self.data['user']['timezone'])

        self.events = []
        with click.progressbar(self.data['days'],
                            label='Preparing keen.io events',
                            fill_char=click.style('#', fg='blue')) as days:

            for day in days:
                dt = self._parse_date(day['date'], timezone)

                self.append_event(dt, 'total', {
                    'seconds': day['grand_total']['total_seconds'],
                })

                categories = [
                    'editor',
                    'language',
                    'operating_system',
                    'project',
                ]
                for category in categories:
                    for item in day.get(category + 's', []):
                        self.append_event(dt, category, {
                            'seconds': item['total_seconds'],
                            'name': item['name'],
                        })

                files = {}
                for project in day.get('projects', []):
                    for f in project.get('dump', {}).get('entities', []):
                        if f['name'] not in files:
                            files[f['name']] = 0
                        files[f['name']] += f['total_seconds']

                for name, seconds in files.items():
                    self.append_event(dt, 'file', {
                        'seconds': seconds,
                        'name': name,
                    })

        if len(self.events) == 0:
            click.echo('No events available for exporting to keen.io')
            return

        click.echo('Uploading events to keen.io...')
        start = datetime.utcfromtimestamp(self.data['range']['start'])
        end = datetime.utcfromtimestamp(self.data['range']['end'])
        collection = 'WakaTime Data Dump from {start} until {end}'.format(
            start=start.strftime('%Y-%m-%d'),
            end=end.strftime('%Y-%m-%d'),
        )
        keen_client.add_events({
            collection: self.events,
        })

    def _parse_date(self, date_str, timezone):
        dt = None
        date_str = date_str.replace('/', '-')
        try:
            dt = datetime.strptime(date_str, '%m-%d-%Y')
        except ValueError:
            dt = datetime.strptime(date_str, '%Y-%m-%d')
        dt = dt.replace(tzinfo=timezone).replace(hour=12)
        return dt
